//---
//
// License: MIT
//
// File: ossim-foo.cpp
//
// Description: Contains application definition "ossim-foo" app.
//
// NOTE:  This is supplied for simple quick test. DO NOT checkin your test to
//        the svn repository.  Simply edit ossim-foo.cpp and run your test.
//        After completion you can do a "git checkout -- ossimfoo.cpp" if
//        you want to keep your working repository up to snuff.
//
// $Id$
//---

// System includes:
#include <cmath>
#include <memory>
#include <string>
#include <sstream>
#include <iostream>
#include <iomanip>
#include <cstring>

// Put your includes here:
#include <ossim/base/ossimString.h>
#include <ossim/base/ossimCommon.h>

using namespace std;

int main(int argc, char *argv[])
{
   return 0;
}
